@@ -32,16 +32,21 @@ struct backend {
 	char			*fqdn;
 	char			*ipaddr;
 	char			*ethaddr;
+	int			ofidx;
+	char			*oface;
 	char			*port;
+	char			*srcaddr;
 	int			weight;
 	int			priority;
 	int			mark;
 	int			estconnlimit;
+	char		*estconnlimit_logprefix;
 	int			state;
 };
 
 void backend_s_print(struct farm *f);
-struct backend * backend_lookup_by_name(struct farm *f, const char *name);
+struct backend * backend_lookup_by_key(struct farm *f, int key, const char *name, int value);
+int backend_validate(struct backend *b);
 int backend_is_available(struct backend *b);
 
 int backend_set_action(struct backend *b, int action);
@@ -57,7 +62,10 @@ int backend_s_find_ethers(struct farm *f);
 
 struct backend * backend_get_first(struct farm *f);
 
+int backend_changed(struct config_pair *c);
 int bck_pre_actionable(struct config_pair *c);
-int bck_pos_actionable(struct config_pair *c);
+int bck_pos_actionable(struct config_pair *c, int action);
+
+int backend_s_gen_priority(struct farm *f);
 
 #endif /* _BACKENDS_H_ */
