@@ -25,17 +25,12 @@
 #include "list.h"
 #include "config.h"
 
-enum families {
-	VALUE_FAMILY_IPV4,
-	VALUE_FAMILY_IPV6,
-	VALUE_FAMILY_INET,
-};
-
 enum modes {
 	VALUE_MODE_SNAT,
 	VALUE_MODE_DNAT,
 	VALUE_MODE_DSR,
 	VALUE_MODE_STLSDNAT,
+	VALUE_MODE_LOCAL,
 };
 
 enum protocols {
@@ -85,19 +80,29 @@ enum states {
 #define VALUE_META_DSTPORT	(1 << 3)
 #define VALUE_META_SRCMAC		(1 << 4)
 #define VALUE_META_DSTMAC		(1 << 5)
+#define VALUE_META_MARK		(1 << 6)
 
 #define VALUE_LOG_NONE			0
 #define VALUE_LOG_INPUT			(1 << 0)
 #define VALUE_LOG_FORWARD		(1 << 1)
 #define VALUE_LOG_OUTPUT		(1 << 2)
 
-#define VALUE_RLD_NONE					0
-#define VALUE_RLD_NEWRTLIMIT_START		(1 << 0)
-#define VALUE_RLD_NEWRTLIMIT_STOP		(1 << 1)
-#define VALUE_RLD_RSTRTLIMIT_START		(1 << 2)
-#define VALUE_RLD_RSTRTLIMIT_STOP		(1 << 3)
-#define VALUE_RLD_ESTCONNLIMIT_START	(1 << 4)
-#define VALUE_RLD_ESTCONNLIMIT_STOP		(1 << 5)
+#define VALUE_RLD_NONE						0
+
+#define VALUE_RLD_NEWRTLIMIT_START			(1 << 0)
+#define VALUE_RLD_RSTRTLIMIT_START			(1 << 1)
+#define VALUE_RLD_ESTCONNLIMIT_START		(1 << 2)
+#define VALUE_RLD_TCPSTRICT_START			(1 << 3)
+#define VALUE_RLD_QUEUE_START				(1 << 4)
+
+#define VALUE_RLD_NEWRTLIMIT_STOP			(1 << 5)
+#define VALUE_RLD_RSTRTLIMIT_STOP			(1 << 6)
+#define VALUE_RLD_ESTCONNLIMIT_STOP			(1 << 7)
+#define VALUE_RLD_TCPSTRICT_STOP			(1 << 8)
+#define VALUE_RLD_QUEUE_STOP				(1 << 9)
+
+#define STATEFUL_RLD_START(x)				(x & VALUE_RLD_NEWRTLIMIT_START) || (x & VALUE_RLD_RSTRTLIMIT_START) || (x & VALUE_RLD_ESTCONNLIMIT_START) || (x & VALUE_RLD_TCPSTRICT_START) || (x & VALUE_RLD_QUEUE_START)
+#define STATEFUL_RLD_STOP(x)				(x & VALUE_RLD_NEWRTLIMIT_STOP) || (x & VALUE_RLD_RSTRTLIMIT_STOP) || (x & VALUE_RLD_ESTCONNLIMIT_STOP) || (x & VALUE_RLD_TCPSTRICT_STOP) || (x & VALUE_RLD_QUEUE_STOP)
 
 struct farm {
 	struct list_head	list;
@@ -116,6 +121,7 @@ struct farm {
 	char			*srcaddr;
 	int			family;
 	int			mode;
+	int			responsettl;
 	int			protocol;
 	int			scheduler;
 	int			schedparam;
@@ -123,34 +129,50 @@ struct farm {
 	int			persistttl;
 	int			helper;
 	int			log;
+	char		*logprefix;
 	int			mark;
 	int			state;
 	int			priority;
 	int			newrtlimit;
 	int			newrtlimitbst;
+	char		*newrtlimit_logprefix;
 	int			rstrtlimit;
 	int			rstrtlimitbst;
+	char		*rstrtlimit_logprefix;
 	int			estconnlimit;
+	char		*estconnlimit_logprefix;
 	int			tcpstrict;
+	char		*tcpstrict_logprefix;
 	int			queue;
+	int			flow_offload;
 	int			total_weight;
 	int			total_bcks;
 	int			bcks_available;
 	int			bcks_are_marked;
 	int			bcks_have_port;
+	int			bcks_have_srcaddr;
+	int			bcks_have_if;
 	int			policies_action;
 	int			policies_used;
+	int			nft_chains;
 	struct list_head	backends;
 	struct list_head	policies;
+	int					total_timed_sessions;
+	struct list_head	timed_sessions;
+	int					total_static_sessions;
+	struct list_head	static_sessions;
 };
 
 struct list_head * farm_s_get_head(void);
+int farm_set_priority(struct farm *f, int new_value);
 void farm_s_print(void);
 int farm_is_ingress_mode(struct farm *f);
 int farm_needs_policies(struct farm *f);
+int farm_needs_flowtable(struct farm *f);
 int farm_set_ifinfo(struct farm *f, int key);
 struct farm * farm_lookup_by_name(const char *name);
 
+int farm_changed(struct config_pair *c);
 int farm_pre_actionable(struct config_pair *c);
 int farm_pos_actionable(struct config_pair *c);
 
