@@ -34,6 +34,7 @@
 #define DEFAULT_PORT		""
 #define DEFAULT_FAMILY		VALUE_FAMILY_IPV4
 #define DEFAULT_MODE		VALUE_MODE_SNAT
+#define DEFAULT_RESPONSETTL	60
 #define DEFAULT_PROTO		VALUE_PROTO_TCP
 #define DEFAULT_SCHED		VALUE_SCHED_RR
 #define DEFAULT_SCHEDPARAM	VALUE_META_NONE
@@ -41,6 +42,8 @@
 #define DEFAULT_PERSISTTM	60
 #define DEFAULT_HELPER		VALUE_HELPER_NONE
 #define DEFAULT_LOG			VALUE_LOG_NONE
+#define DEFAULT_LOG_LOGPREFIX	"TYPE-FNAME "
+#define DEFAULT_LOGPREFIX	"KNAME-FNAME "
 #define DEFAULT_MARK		0x0
 #define DEFAULT_WEIGHT		1
 #define DEFAULT_PRIORITY	1
@@ -51,12 +54,25 @@
 #define DEFAULT_RTLIMITBURST	0
 #define DEFAULT_RSTRTLIMIT	0
 #define DEFAULT_ESTCONNLIMIT	0
+#define DEFAULT_B_ESTCONNLIMIT_LOGPREFIX	"KNAME-FNAME-BNAME "
 #define DEFAULT_TCPSTRICT	VALUE_SWITCH_OFF
 #define DEFAULT_QUEUE		-1
+#define DEFAULT_FLOWOFFLOAD		0
+
 #define DEFAULT_POLICY_TYPE	VALUE_TYPE_BLACK
 #define DEFAULT_POLICY_TIMEOUT	0
-#define DEFAULT_POLICY_PRIORITY	-1
-#define DEFAULT_ELEMENT_TIME		""
+#define DEFAULT_POLICY_PRIORITY	1
+#define DEFAULT_POLICY_LOGPREFIX	"KNAME-TYPE-PNAME-FNAME "
+#define DEFAULT_ELEMENT_TIME			NULL
+#define DEFAULT_SESSION_EXPIRATION		NULL
+
+#define UNDEFINED_VALUE					"UNDEFINED"
+#define IFACE_LOOPBACK					"lo"
+
+enum obj_start {
+	OBJ_START,
+	OBJ_START_INV
+};
 
 enum levels {
 	LEVEL_INIT,
@@ -65,11 +81,13 @@ enum levels {
 	LEVEL_FARMPOLICY,
 	LEVEL_POLICIES,
 	LEVEL_ELEMENTS,
+	LEVEL_SESSIONS,
 };
 
 enum actions {
 	ACTION_START,
 	ACTION_STOP,
+	ACTION_FLUSH,
 	ACTION_RELOAD,
 	ACTION_DELETE,
 	ACTION_NONE,
@@ -116,6 +134,24 @@ enum keys {
 	KEY_TIMEOUT,
 	KEY_DATA,
 	KEY_TIME,
+	KEY_LOGPREFIX,
+	KEY_NEWRTLIMIT_LOGPREFIX,
+	KEY_RSTRTLIMIT_LOGPREFIX,
+	KEY_ESTCONNLIMIT_LOGPREFIX,
+	KEY_TCPSTRICT_LOGPREFIX,
+	KEY_RESPONSETTL,
+	KEY_FLOWOFFLOAD,
+	KEY_SESSIONS,
+	KEY_CLIENT,
+	KEY_BACKEND,
+	KEY_USED,
+};
+
+enum families {
+	VALUE_FAMILY_IPV4,
+	VALUE_FAMILY_IPV6,
+	VALUE_FAMILY_INET,
+	VALUE_FAMILY_NETDEV,
 };
 
 struct obj_config {
@@ -124,6 +160,7 @@ struct obj_config {
 	struct policy		*pptr;
 	struct element		*eptr;
 	struct farmpolicy	*fpptr;
+	struct session		*sptr;
 	struct config_pair	*c;
 };
 
@@ -135,7 +172,20 @@ int obj_get_dsr_counter(void);
 void obj_set_dsr_counter(int new_value);
 
 struct obj_config * obj_get_current_object(void);
+struct farm * obj_get_current_farm(void);
+struct backend * obj_get_current_backend(void);
+struct policy * obj_get_current_policy(void);
+struct farmpolicy * obj_get_current_farmpolicy(void);
+struct element * obj_get_current_element(void);
+struct session * obj_get_current_session(void);
+void obj_set_current_farm(struct farm *f);
+void obj_set_current_backend(struct backend *b);
+void obj_set_current_policy(struct policy *p);
+void obj_set_current_farmpolicy(struct farmpolicy *fp);
+void obj_set_current_element(struct element *e);
+void obj_set_current_session(struct session *s);
 
+char * obj_print_key(int key);
 char * obj_print_family(int family);
 char * obj_print_mode(int mode);
 char * obj_print_proto(int protocol);
@@ -147,8 +197,11 @@ char * obj_print_state(int state);
 char * obj_print_switch(int value);
 int obj_set_attribute(struct config_pair *c, int actionable);
 int obj_set_attribute_string(char *src, char **dst);
+int obj_equ_attribute_string(char *stra, char *strb);
+void obj_set_attribute_int(int *src, int value);
+int obj_equ_attribute_int(int valuea, int valueb);
 void obj_print(void);
-int obj_rulerize(void);
+int obj_rulerize(int mode);
 
 struct list_head * obj_get_policies(void);
 int obj_get_total_policies(void);
