@@ -23,12 +23,31 @@
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
+#include <net/if.h>
 
 #include "backends.h"
 #include "farms.h"
 #include "objects.h"
 #include "network.h"
+#include "sessions.h"
+#include "nft.h"
 
+static int backend_s_set_marked(struct farm *f)
+{
+	struct backend *b;
+
+	syslog(LOG_DEBUG, "%s():%d: finding marked backends for %s", __FUNCTION__, __LINE__, f->name);
+
+	list_for_each_entry(b, &f->backends, list) {
+		if (b->mark != DEFAULT_MARK) {
+			f->bcks_are_marked = 1;
+			return 1;
+		}
+	}
+
+	f->bcks_are_marked = 0;
+	return 0;
+}
 
 static struct backend * backend_create(struct farm *f, char *name)
 {
@@ -43,17 +62,22 @@ static struct backend * backend_create(struct farm *f, char *name)
 
 	b->fqdn = DEFAULT_FQDN;
 	b->ethaddr = DEFAULT_ETHADDR;
+	b->oface = DEFAULT_IFNAME;
+	b->ofidx = DEFAULT_IFIDX;
 	b->ipaddr = DEFAULT_IPADDR;
 	b->port = DEFAULT_PORT;
+	b->srcaddr = DEFAULT_SRCADDR;
 	b->weight = DEFAULT_WEIGHT;
 	b->priority = DEFAULT_PRIORITY;
 	b->mark = DEFAULT_MARK;
 	b->estconnlimit = DEFAULT_ESTCONNLIMIT;
+	b->estconnlimit_logprefix = DEFAULT_B_ESTCONNLIMIT_LOGPREFIX;
 	b->state = DEFAULT_BACKEND_STATE;
 	b->action = DEFAULT_ACTION;
 
 	list_add_tail(&b->list, &f->backends);
 	f->total_bcks++;
+	backend_s_set_marked(f);
 
 	return b;
 }
@@ -65,12 +89,18 @@ static int backend_delete_node(struct backend *b)
 		free(b->name);
 	if (b->fqdn && strcmp(b->fqdn, "") != 0)
 		free(b->fqdn);
+	if (b->oface && strcmp(b->oface, "") != 0)
+		free(b->oface);
 	if (b->ipaddr && strcmp(b->ipaddr, "") != 0)
 		free(b->ipaddr);
 	if (b->ethaddr && strcmp(b->ethaddr, "") != 0)
 		free(b->ethaddr);
 	if (b->port && strcmp(b->port, "") != 0)
 		free(b->port);
+	if (b->srcaddr && strcmp(b->srcaddr, "") != 0)
+		free(b->srcaddr);
+	if (b->estconnlimit_logprefix && strcmp(b->estconnlimit_logprefix, DEFAULT_B_ESTCONNLIMIT_LOGPREFIX) != 0)
+		free(b->estconnlimit_logprefix);
 
 	free(b);
 
@@ -79,7 +109,13 @@ static int backend_delete_node(struct backend *b)
 
 static int backend_delete(struct backend *b)
 {
-	// TODO: Stop backend && reload farm && rulerize()
+	struct farm *f = b->parent;
+	backend_set_action(b, ACTION_STOP);
+
+	if (f->priority >= 1 && b->priority <= f->priority) {
+		farm_set_priority(f, f->priority-1);
+		obj_rulerize(OBJ_START);
+	}
 
 	backend_delete_node(b);
 
@@ -97,6 +133,11 @@ void backend_s_print(struct farm *f)
 		if (b->fqdn)
 			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_FQDN, b->fqdn);
 
+		if (b->oface)
+			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_OFACE, b->oface);
+
+		syslog(LOG_DEBUG,"      *[ofidx] %d", b->ofidx);
+
 		if (b->ipaddr)
 			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_IPADDR, b->ipaddr);
 
@@ -106,8 +147,14 @@ void backend_s_print(struct farm *f)
 		if (b->port)
 			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_PORT, b->port);
 
+		if (b->srcaddr)
+			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_SRCADDR, b->srcaddr);
+
 		syslog(LOG_DEBUG,"       [%s] 0x%x", CONFIG_KEY_MARK, b->mark);
 		syslog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_ESTCONNLIMIT, b->estconnlimit);
+		if (b->estconnlimit_logprefix && strcmp(b->estconnlimit_logprefix, DEFAULT_B_ESTCONNLIMIT_LOGPREFIX) != 0)
+			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_ESTCONNLIMIT_LOGPREFIX, b->estconnlimit_logprefix);
+
 		syslog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_WEIGHT, b->weight);
 		syslog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_PRIORITY, b->priority);
 		syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_STATE, obj_print_state(b->state));
@@ -115,13 +162,35 @@ void backend_s_print(struct farm *f)
 	}
 }
 
-struct backend * backend_lookup_by_name(struct farm *f, const char *name)
+struct backend * backend_lookup_by_key(struct farm *f, int key, const char *name, int value)
 {
 	struct backend *b;
+	int bckmark;
+
+	syslog(LOG_DEBUG, "%s():%d: farm %s key %d name %s value %d", __FUNCTION__, __LINE__, f->name, key, name, value);
 
 	list_for_each_entry(b, &f->backends, list) {
-		if (strcmp(b->name, name) == 0)
-			return b;
+		switch (key) {
+		case KEY_NAME:
+			if (strcmp(b->name, name) == 0)
+				return b;
+			break;
+		case KEY_MARK:
+			bckmark = get_bck_mark(b);
+			if (bckmark == value)
+				return b;
+			break;
+		case KEY_ETHADDR:
+			if (b->ethaddr && strcmp(b->ethaddr, name) == 0)
+				return b;
+			break;
+		case KEY_IPADDR:
+			if (b->ipaddr && strcmp(b->ipaddr, name) == 0)
+				return b;
+			break;
+		default:
+			return NULL;
+		}
 	}
 
 	return NULL;
@@ -134,6 +203,8 @@ static int backend_set_ipaddr_from_ether(struct backend *b)
 	unsigned char dst_ethaddr[ETH_HW_ADDR_LEN];
 	unsigned char src_ethaddr[ETH_HW_ADDR_LEN];
 	char streth[ETH_HW_STR_LEN] = {};
+	int *oface;
+	char **source_ip;
 
 	if (!farm_is_ingress_mode(f))
 		return 0;
@@ -145,7 +216,28 @@ static int backend_set_ipaddr_from_ether(struct backend *b)
 
 	sscanf(f->iethaddr, "%hhx:%hhx:%hhx:%hhx:%hhx:%hhx", &src_ethaddr[0], &src_ethaddr[1], &src_ethaddr[2], &src_ethaddr[3], &src_ethaddr[4], &src_ethaddr[5]);
 
-	ret = net_get_neigh_ether((unsigned char **) &dst_ethaddr, src_ethaddr, f->family, f->virtaddr, b->ipaddr, f->ofidx);
+	oface = &f->ifidx;
+	source_ip = &f->virtaddr;
+
+	if (f->srcaddr != DEFAULT_SRCADDR)
+		source_ip = &f->srcaddr;
+
+	ret = net_get_neigh_ether((unsigned char **) &dst_ethaddr, src_ethaddr, f->family, *source_ip, b->ipaddr, *oface);
+
+	if (ret != 0) {
+		oface = &f->ofidx;
+
+		if (b->ofidx != DEFAULT_IFIDX)
+			oface = &b->ofidx;
+
+		if (f->srcaddr != DEFAULT_SRCADDR)
+			source_ip = &f->srcaddr;
+
+		if (b->srcaddr != DEFAULT_SRCADDR)
+			source_ip = &b->srcaddr;
+
+		ret = net_get_neigh_ether((unsigned char **) &dst_ethaddr, src_ethaddr, f->family, *source_ip, b->ipaddr, *oface);
+	}
 
 	if (ret == 0) {
 		sprintf(streth, "%02x:%02x:%02x:%02x:%02x:%02x", dst_ethaddr[0],
@@ -190,53 +282,38 @@ static int backend_set_estconnlimit(struct backend *b, int new_value)
 	return 0;
 }
 
+static void backend_s_update_counters(struct farm *f)
+{
+	struct backend *bp, *next;
+
+	syslog(LOG_DEBUG, "%s():%d: farm %s", __FUNCTION__, __LINE__, f->name);
+
+	f->bcks_available = 0;
+	f->total_weight = 0;
+
+	list_for_each_entry_safe(bp, next, &f->backends, list) {
+		if (backend_is_available(bp)) {
+			f->bcks_available++;
+			f->total_weight += bp->weight;
+		}
+	}
+}
+
 static int backend_set_priority(struct backend *b, int new_value)
 {
-	struct farm *f = b->parent;
 	int old_value = b->priority;
 
 	syslog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
 	       __FUNCTION__, __LINE__, old_value, new_value);
 
-	if (backend_is_available(b) &&
-	    new_value > f->priority) {
-		f->bcks_available--;
-		if (f->bcks_available < 0)
-			f->bcks_available = 0;
-
-		f->total_weight -= b->weight;
-		if (f->total_weight < 0)
-			f->total_weight = 0;
-	}
-
-	else if (old_value > f->priority &&
-		 backend_is_available(b)) {
-		f->bcks_available++;
-		f->total_weight += b->weight;
-	}
+	if (new_value <= 0)
+		return -1;
 
 	b->priority = new_value;
 
 	return 0;
 }
 
-static int backend_s_set_marked(struct farm *f)
-{
-	struct backend *b;
-
-	syslog(LOG_DEBUG, "%s():%d: finding marked backends for %s", __FUNCTION__, __LINE__, f->name);
-
-	list_for_each_entry(b, &f->backends, list) {
-		if (b->mark != DEFAULT_MARK) {
-			f->bcks_are_marked = 1;
-			return 1;
-		}
-	}
-
-	f->bcks_are_marked = 0;
-	return 0;
-}
-
 static int backend_s_set_ports(struct farm *f)
 {
 	struct backend *b;
@@ -254,6 +331,23 @@ static int backend_s_set_ports(struct farm *f)
 	return 0;
 }
 
+static int backend_s_set_srcaddr(struct farm *f)
+{
+	struct backend *b;
+
+	syslog(LOG_DEBUG, "%s():%d: finding backends with srouce address for %s", __FUNCTION__, __LINE__, f->name);
+
+	list_for_each_entry(b, &f->backends, list) {
+		if (b->srcaddr && strcmp(b->srcaddr, "") != 0) {
+			f->bcks_have_srcaddr = 1;
+			return 1;
+		}
+	}
+
+	f->bcks_have_srcaddr = 0;
+	return 0;
+}
+
 static int backend_set_mark(struct backend *b, int new_value)
 {
 	int old_value = b->mark;
@@ -288,6 +382,76 @@ static int backend_set_port(struct backend *b, char *new_value)
 	return 0;
 }
 
+static int backend_set_srcaddr(struct backend *b, char *new_value)
+{
+	char *old_value = b->srcaddr;
+
+	syslog(LOG_DEBUG, "%s():%d: current value is %s, but new value will be %s",
+	       __FUNCTION__, __LINE__, old_value, new_value);
+
+	obj_set_attribute_string(new_value, &b->srcaddr);
+
+	if (b->srcaddr && strcmp(b->srcaddr, "") != 0)
+		b->parent->bcks_have_srcaddr = 1;
+	else
+		backend_s_set_srcaddr(b->parent);
+
+	return 0;
+}
+
+static int backend_set_ifinfo(struct backend *b, int key)
+{
+	struct farm *f = b->parent;
+	char if_str[IFNAMSIZ];
+	int if_index;
+	int ret = 0;
+
+	syslog(LOG_DEBUG, "%s():%d: backend %s set interface info for interface key %d", __FUNCTION__, __LINE__, b->name, key);
+
+	if (!farm_is_ingress_mode(f)) {
+		syslog(LOG_DEBUG, "%s():%d: farm %s is not in ingress mode", __FUNCTION__, __LINE__, f->name);
+		return 0;
+	}
+
+	if (f->oface && strcmp(f->oface, IFACE_LOOPBACK) == 0) {
+		syslog(LOG_DEBUG, "%s():%d: backend %s in farm %s doesn't require output netinfo, loopback interface", __FUNCTION__, __LINE__, b->name, f->name);
+		f->ifidx = 0;
+		return 0;
+	}
+
+	switch (key) {
+	case KEY_OFACE:
+		if (!b || b->ipaddr == DEFAULT_IPADDR) {
+			syslog(LOG_ERR, "%s():%d: there is no backend yet in the farm %s", __FUNCTION__, __LINE__, f->name);
+			return 0;
+		}
+
+		ret = net_get_local_ifidx_per_remote_host(b->ipaddr, &if_index);
+		if (ret == -1) {
+			syslog(LOG_ERR, "%s():%d: unable to get the outbound interface to %s for the backend %s in farm %s", __FUNCTION__, __LINE__, b->ipaddr, b->name, f->name);
+			return -1;
+		}
+
+		if (f->ofidx != if_index) {
+			f->bcks_have_if = 1;
+			b->ofidx = if_index;
+		}
+
+		if (if_indextoname(if_index, if_str) == NULL) {
+			syslog(LOG_ERR, "%s():%d: unable to get the outbound interface name with index %d required by the backend %s in farm %s", __FUNCTION__, __LINE__, if_index, b->name, f->name);
+			return -1;
+		}
+
+		if (strcmp(f->oface, if_str) != 0) {
+			obj_set_attribute_string(if_str, &b->oface);
+			net_strim_netface(b->oface);
+		}
+		break;
+	}
+
+	return 0;
+}
+
 static int backend_set_ipaddr(struct backend *b, char *new_value)
 {
 	char *old_value = b->ipaddr;
@@ -298,19 +462,92 @@ static int backend_set_ipaddr(struct backend *b, char *new_value)
 	obj_set_attribute_string(new_value, &b->ipaddr);
 	obj_set_attribute_string("", &b->ethaddr);
 
-	if (farm_set_ifinfo(b->parent, KEY_OFACE) == -1 ||
-	    backend_set_ipaddr_from_ether(b) == -1) {
-		syslog(LOG_DEBUG, "%s():%d: backend %s comes to OFF", __FUNCTION__, __LINE__, b->name);
-		backend_set_state(b, VALUE_STATE_CONFERR);
-	} else {
-		if (b->state == VALUE_STATE_CONFERR)
+	if (farm_set_ifinfo(b->parent, KEY_OFACE) == 0 &&
+		backend_set_ifinfo(b, KEY_OFACE) == 0 &&
+	    backend_set_ipaddr_from_ether(b) == 0 ) {
+		if (old_value != DEFAULT_IPADDR && b->state == VALUE_STATE_CONFERR)
 			backend_set_state(b, VALUE_STATE_UP);
+	} else {
+		syslog(LOG_DEBUG, "%s():%d: backend %s comes to OFF", __FUNCTION__, __LINE__, b->name);
+		if (old_value != DEFAULT_IPADDR)
+			backend_set_state(b, VALUE_STATE_CONFERR);
 	}
 
 	return 0;
 }
 
-static int backend_validate(struct backend *b)
+static int backend_is_usable(struct backend *b)
+{
+	struct farm *f = b->parent;
+
+	syslog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d",
+	       __FUNCTION__, __LINE__, b->name, obj_print_state(b->state), b->priority);
+
+	return (b->state == VALUE_STATE_UP) &&
+			(b->priority <= f->priority);
+}
+
+int backend_changed(struct config_pair *c)
+{
+	struct farm *f = obj_get_current_farm();
+	struct backend *b = obj_get_current_backend();
+
+	if (!f || !b)
+		return -1;
+
+	syslog(LOG_DEBUG, "%s():%d: farm %s backend %s with param %d", __FUNCTION__, __LINE__, f->name, b->name, c->key);
+
+	switch (c->key) {
+	case KEY_NAME:
+		return 1;
+		break;
+	case KEY_NEWNAME:
+		return !obj_equ_attribute_string(b->name, c->str_value);
+		break;
+	case KEY_FQDN:
+		return !obj_equ_attribute_string(b->fqdn, c->str_value);
+		break;
+	case KEY_IPADDR:
+		return !obj_equ_attribute_string(b->ipaddr, c->str_value);
+		break;
+	case KEY_ETHADDR:
+		return !obj_equ_attribute_string(b->ethaddr, c->str_value);
+		break;
+	case KEY_PORT:
+		return !obj_equ_attribute_string(b->port, c->str_value);
+		break;
+	case KEY_SRCADDR:
+		return !obj_equ_attribute_string(b->srcaddr, c->str_value);
+		break;
+	case KEY_WEIGHT:
+		return !obj_equ_attribute_int(b->weight, c->int_value);
+		break;
+	case KEY_PRIORITY:
+		return !obj_equ_attribute_int(b->priority, c->int_value);
+		break;
+	case KEY_MARK:
+		return !obj_equ_attribute_int(b->mark, c->int_value);
+		break;
+	case KEY_STATE:
+		return !obj_equ_attribute_int(b->state, c->int_value);
+		break;
+	case KEY_ACTION:
+		return !obj_equ_attribute_int(b->action, c->int_value);
+		break;
+	case KEY_ESTCONNLIMIT:
+		return !obj_equ_attribute_int(b->estconnlimit, c->int_value);
+		break;
+	case KEY_ESTCONNLIMIT_LOGPREFIX:
+		return !obj_equ_attribute_string(b->estconnlimit_logprefix, c->str_value);
+		break;
+	default:
+		break;
+	}
+
+	return 0;
+}
+
+int backend_validate(struct backend *b)
 {
 	struct farm *f = b->parent;
 
@@ -327,17 +564,6 @@ static int backend_validate(struct backend *b)
 	return 1;
 }
 
-static int backend_is_usable(struct backend *b)
-{
-	struct farm *f = b->parent;
-
-	syslog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d",
-	       __FUNCTION__, __LINE__, b->name, obj_print_state(b->state), b->priority);
-
-	return (b->state == VALUE_STATE_UP) &&
-			(b->priority <= f->priority);
-}
-
 int backend_is_available(struct backend *b)
 {
 	syslog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d",
@@ -360,17 +586,18 @@ int backend_set_action(struct backend *b, int action)
 	}
 
 	if (action == ACTION_STOP) {
-		if (b->state == VALUE_STATE_UP)
+		if (backend_is_available(b))
 		{
 			b->action = action;
 			is_actionated = 1;
 		}
 		backend_set_state(b, VALUE_STATE_OFF);
+
 		return is_actionated;
 	}
 
 	if (action == ACTION_START) {
-		if (b->state != VALUE_STATE_UP)
+		if (!backend_is_available(b))
 		{
 			b->action = action;
 			is_actionated = 1;
@@ -402,11 +629,12 @@ int backend_s_delete(struct farm *f)
 	struct backend *b, *next;
 
 	list_for_each_entry_safe(b, next, &f->backends, list)
-		backend_delete(b);
+		backend_delete_node(b);
 
 	f->total_bcks = 0;
 	f->bcks_available = 0;
 	f->total_weight = 0;
+	f->bcks_have_if = 0;
 
 	return 0;
 }
@@ -427,26 +655,21 @@ int backend_s_validate(struct farm *f)
 
 int backend_set_attribute(struct config_pair *c)
 {
-	struct obj_config *cur = obj_get_current_object();
-	struct backend *b;
+	struct farm *f = obj_get_current_farm();
+	struct backend *b = obj_get_current_backend();
 
-	if (!cur->fptr)
+	if (!f || (c->key != KEY_NAME && !b))
 		return PARSER_OBJ_UNKNOWN;
 
-	if (c->key != KEY_NAME && !cur->bptr)
-		return PARSER_OBJ_UNKNOWN;
-
-	b = cur->bptr;
-
 	switch (c->key) {
 	case KEY_NAME:
-		b = backend_lookup_by_name(cur->fptr, c->str_value);
+		b = backend_lookup_by_key(f, KEY_NAME, c->str_value, 0);
 		if (!b) {
-			b = backend_create(cur->fptr, c->str_value);
+			b = backend_create(f, c->str_value);
 			if (!b)
 				return -1;
 		}
-		cur->bptr = b;
+		obj_set_current_backend(b);
 		break;
 	case KEY_NEWNAME:
 		obj_set_attribute_string(c->str_value, &b->name);
@@ -463,6 +686,9 @@ int backend_set_attribute(struct config_pair *c)
 	case KEY_PORT:
 		backend_set_port(b, c->str_value);
 		break;
+	case KEY_SRCADDR:
+		backend_set_srcaddr(b, c->str_value);
+		break;
 	case KEY_WEIGHT:
 		backend_set_weight(b, c->int_value);
 		break;
@@ -473,7 +699,10 @@ int backend_set_attribute(struct config_pair *c)
 		backend_set_mark(b, c->int_value);
 		break;
 	case KEY_STATE:
-		backend_set_state(b, c->int_value);
+		if (c->int_value == VALUE_STATE_CONFERR)
+			backend_set_state(b, VALUE_STATE_UP);
+		else
+			backend_set_state(b, c->int_value);
 		break;
 	case KEY_ESTCONNLIMIT:
 		backend_set_estconnlimit(b, c->int_value);
@@ -481,6 +710,9 @@ int backend_set_attribute(struct config_pair *c)
 	case KEY_ACTION:
 		backend_set_action(b, c->int_value);
 		break;
+	case KEY_ESTCONNLIMIT_LOGPREFIX:
+		obj_set_attribute_string(c->str_value, &b->estconnlimit_logprefix);
+		break;
 	default:
 		return -1;
 	}
@@ -488,30 +720,23 @@ int backend_set_attribute(struct config_pair *c)
 	return PARSER_OK;
 }
 
-static int backend_switch(struct backend *b, int new_state)
+static int backend_switch(struct backend *b)
 {
 	struct farm *f = b->parent;
 
-	syslog(LOG_DEBUG, "%s():%d: backend %s switched to %s",
-	       __FUNCTION__, __LINE__, b->name, obj_print_state(new_state));
+	syslog(LOG_DEBUG, "%s():%d: backend %s with state %s switched",
+	       __FUNCTION__, __LINE__, b->name, obj_print_state(b->state));
 
-	if (b->state == VALUE_STATE_UP) {
-		f->total_weight += b->weight;
-		f->bcks_available++;
+	if (f->persistence != VALUE_META_NONE)
+		session_backend_action(f, b, b->state);
+
+	if (b->state == VALUE_STATE_UP)
 		b->action = ACTION_START;
-		farm_set_action(f, ACTION_RELOAD);
-	} else {
-		f->total_weight -= b->weight;
-		if (f->total_weight < 0)
-			f->total_weight = 0;
-
-		f->bcks_available--;
-		if (f->bcks_available < 0)
-			f->bcks_available = 0;
-
+	else
 		b->action = ACTION_STOP;
-		farm_set_action(f, ACTION_RELOAD);
-	}
+
+	farm_set_action(f, ACTION_RELOAD);
+	backend_s_update_counters(f);
 
 	return 0;
 }
@@ -534,12 +759,12 @@ int backend_set_state(struct backend *b, int new_value)
 			return 0;
 		}
 		if (backend_is_usable(b))
-			backend_switch(b, new_value);
+			backend_switch(b);
 		break;
 	default:
 		if (backend_is_usable(b)) {
 			b->state = new_value;
-			backend_switch(b, new_value);
+			backend_switch(b);
 		} else
 			b->state = new_value;
 		break;
@@ -601,81 +826,76 @@ struct backend * backend_get_first(struct farm *f)
 
 int bck_pre_actionable(struct config_pair *c)
 {
-	struct obj_config *cur = obj_get_current_object();
-	struct farm *f;
-	struct backend *b;
+	struct farm *f = obj_get_current_farm();
+	struct backend *b = obj_get_current_backend();
 
-	if (!cur->fptr || !cur->bptr)
+	if (!f || !b)
 		return -1;
 
-	f = cur->fptr;
-	b = cur->bptr;
-
 	syslog(LOG_DEBUG, "%s():%d: pre actionable backend %s of farm %s with param %d", __FUNCTION__, __LINE__, b->name, f->name, c->key);
 
+	// changing priority of a down backend could affect others, force a farm restart
+	if (b->state != VALUE_STATE_UP && b->state != VALUE_STATE_CONFERR && c->key == KEY_PRIORITY) {
+		farm_set_action(f, ACTION_STOP);
+		farm_rulerize(f);
+		return ACTION_FLUSH;
+	}
+
 	if (b->state != VALUE_STATE_UP && c->key != KEY_STATE)
-		return 1;
+		return ACTION_NONE;
 
 	switch (c->key) {
 	case KEY_NAME:
 		break;
+
 	case KEY_ETHADDR:
 	case KEY_IPADDR:
 	case KEY_PORT:
+	case KEY_SRCADDR:
+	case KEY_MARK:
 	case KEY_PRIORITY:
 	case KEY_ESTCONNLIMIT:
-
-		if (backend_set_action(b, ACTION_STOP) &&
-			farm_set_action(f, ACTION_RELOAD)) {
+		if (backend_set_action(b, ACTION_STOP)) {
+			farm_set_action(f, ACTION_RELOAD);
 			farm_rulerize(f);
 		}
-
+		return ACTION_START;
 		break;
+
 	case KEY_STATE:
-	case KEY_MARK:
 	case KEY_WEIGHT:
+	case KEY_ESTCONNLIMIT_LOGPREFIX:
+		return ACTION_RELOAD;
+		break;
+
 	default:
 		break;
 	}
 
-	return 0;
+	return ACTION_NONE;
 }
 
-int bck_pos_actionable(struct config_pair *c)
+int bck_pos_actionable(struct config_pair *c, int action)
 {
-	struct obj_config *cur = obj_get_current_object();
-	struct farm *f;
-	struct backend *b;
+	struct farm *f = obj_get_current_farm();
+	struct backend *b = obj_get_current_backend();
 
-	if (!cur->fptr || !cur->bptr)
+	if (!f || !b)
 		return -1;
 
-	f = cur->fptr;
-	b = cur->bptr;
+	syslog(LOG_DEBUG, "%s():%d: pos actionable backend %s of farm %s with param %d action %d", __FUNCTION__, __LINE__, b->name, f->name, c->key, action);
 
-	syslog(LOG_DEBUG, "%s():%d: pos actionable backend %s of farm %s with param %d", __FUNCTION__, __LINE__, b->name, f->name, c->key);
-
-	switch (c->key) {
-	case KEY_NAME:
+	switch (action) {
+	case ACTION_START:
+		if (backend_set_action(b, ACTION_START))
+			farm_set_action(f, ACTION_RELOAD);
 		break;
-	case KEY_ETHADDR:
-	case KEY_IPADDR:
-	case KEY_PORT:
-	case KEY_PRIORITY:
-	case KEY_ESTCONNLIMIT:
-
-		if (backend_set_action(b, ACTION_START) &&
-			farm_set_action(f, ACTION_RELOAD)) {
-			farm_rulerize(f);
-		}
-
-		break;
-	case KEY_STATE:
-	case KEY_MARK:
-	case KEY_WEIGHT:
-
+	case ACTION_RELOAD:
 		farm_set_action(f, ACTION_RELOAD);
-
+		break;
+	case ACTION_FLUSH:
+		farm_set_action(f, ACTION_START);
+		farm_rulerize(f);
 		break;
 	default:
 		break;
@@ -683,3 +903,32 @@ int bck_pos_actionable(struct config_pair *c)
 
 	return 0;
 }
+
+int backend_s_gen_priority(struct farm *f)
+{
+	struct backend *b, *next;
+	int are_down;
+	int old_prio = f->priority;
+	int new_prio = DEFAULT_PRIORITY;
+
+	syslog(LOG_DEBUG, "%s():%d: farm %s", __FUNCTION__, __LINE__, f->name);
+
+	do {
+		are_down = 0;
+		list_for_each_entry_safe(b, next, &f->backends, list) {
+			if (b->priority == new_prio && b->state != VALUE_STATE_UP)
+				are_down++;
+			if (b->priority > f->priority && b->priority == new_prio && b->state == VALUE_STATE_UP)
+				backend_set_action(b, ACTION_START);
+		}
+		new_prio += are_down;
+	} while (are_down);
+
+	f->priority = new_prio;
+
+	syslog(LOG_DEBUG, "%s():%d: priority is %d", __FUNCTION__, __LINE__, f->priority);
+
+	backend_s_update_counters(f);
+
+	return f->priority != old_prio;
+}
