@@ -27,6 +27,7 @@
 
 #include "farms.h"
 #include "backends.h"
+#include "sessions.h"
 #include "farmpolicy.h"
 #include "objects.h"
 #include "config.h"
@@ -58,6 +59,7 @@ static struct farm * farm_create(char *name)
 	pfarm->srcaddr = DEFAULT_SRCADDR;
 	pfarm->family = DEFAULT_FAMILY;
 	pfarm->mode = DEFAULT_MODE;
+	pfarm->responsettl = DEFAULT_RESPONSETTL;
 	pfarm->protocol = DEFAULT_PROTO;
 	pfarm->scheduler = DEFAULT_SCHED;
 	pfarm->schedparam = DEFAULT_SCHEDPARAM;
@@ -65,6 +67,7 @@ static struct farm * farm_create(char *name)
 	pfarm->persistttl = DEFAULT_PERSISTTM;
 	pfarm->helper = DEFAULT_HELPER;
 	pfarm->log = DEFAULT_LOG;
+	pfarm->logprefix = DEFAULT_LOG_LOGPREFIX;
 	pfarm->mark = DEFAULT_MARK;
 	pfarm->state = DEFAULT_FARM_STATE;
 	pfarm->action = DEFAULT_ACTION;
@@ -77,18 +80,31 @@ static struct farm * farm_create(char *name)
 	pfarm->priority = DEFAULT_PRIORITY;
 	pfarm->newrtlimit = DEFAULT_NEWRTLIMIT;
 	pfarm->newrtlimitbst = DEFAULT_RTLIMITBURST;
+	pfarm->newrtlimit_logprefix = DEFAULT_LOGPREFIX;
 	pfarm->rstrtlimit = DEFAULT_RSTRTLIMIT;
 	pfarm->rstrtlimitbst = DEFAULT_RTLIMITBURST;
+	pfarm->rstrtlimit_logprefix = DEFAULT_LOGPREFIX;
 	pfarm->estconnlimit = DEFAULT_ESTCONNLIMIT;
+	pfarm->estconnlimit_logprefix = DEFAULT_LOGPREFIX;
 	pfarm->tcpstrict = DEFAULT_TCPSTRICT;
+	pfarm->tcpstrict_logprefix = DEFAULT_LOGPREFIX;
 	pfarm->queue = DEFAULT_QUEUE;
+	pfarm->flow_offload = DEFAULT_FLOWOFFLOAD;
 
 	pfarm->total_bcks = 0;
 	pfarm->bcks_available = 0;
 	pfarm->bcks_are_marked = 0;
 	pfarm->bcks_have_port = 0;
+	pfarm->bcks_have_srcaddr = 0;
+	pfarm->bcks_have_if = 0;
 	pfarm->policies_used = 0;
 	pfarm->policies_action = ACTION_NONE;
+	pfarm->nft_chains = 0;
+
+	init_list_head(&pfarm->static_sessions);
+	pfarm->total_static_sessions = 0;
+	init_list_head(&pfarm->timed_sessions);
+	pfarm->total_timed_sessions = 0;
 
 	list_add_tail(&pfarm->list, farms);
 	obj_set_total_farms(obj_get_total_farms() + 1);
@@ -98,6 +114,9 @@ static struct farm * farm_create(char *name)
 
 static int farm_delete(struct farm *pfarm)
 {
+	syslog(LOG_DEBUG, "%s():%d: deleting farm %s",
+	       __FUNCTION__, __LINE__, pfarm->name);
+
 	backend_s_delete(pfarm);
 	farmpolicy_s_delete(pfarm);
 	list_del(&pfarm->list);
@@ -118,6 +137,16 @@ static int farm_delete(struct farm *pfarm)
 		free(pfarm->virtaddr);
 	if (pfarm->virtports && strcmp(pfarm->virtports, "") != 0)
 		free(pfarm->virtports);
+	if (pfarm->logprefix && strcmp(pfarm->logprefix, DEFAULT_LOG_LOGPREFIX) != 0)
+		free(pfarm->logprefix);
+	if (pfarm->newrtlimit_logprefix && strcmp(pfarm->newrtlimit_logprefix, DEFAULT_LOGPREFIX) != 0)
+		free(pfarm->newrtlimit_logprefix);
+	if (pfarm->rstrtlimit_logprefix && strcmp(pfarm->rstrtlimit_logprefix, DEFAULT_LOGPREFIX) != 0)
+		free(pfarm->rstrtlimit_logprefix);
+	if (pfarm->estconnlimit_logprefix && strcmp(pfarm->estconnlimit_logprefix, DEFAULT_LOGPREFIX) != 0)
+		free(pfarm->estconnlimit_logprefix);
+	if (pfarm->tcpstrict_logprefix && strcmp(pfarm->tcpstrict_logprefix, DEFAULT_LOGPREFIX) != 0)
+		free(pfarm->tcpstrict_logprefix);
 
 	free(pfarm);
 	obj_set_total_farms(obj_get_total_farms() - 1);
@@ -138,13 +167,15 @@ static int farm_validate(struct farm *f)
 		return 0;
 	}
 
-	if (farm_is_ingress_mode(f) &&
-		(!f->iface || (strcmp(f->iface, "") == 0))) {
+	if ((farm_is_ingress_mode(f) || farm_needs_flowtable(f)) &&
+		((!f->iface || (strcmp(f->iface, "") == 0)) ||
+		 (!f->oface || (strcmp(f->oface, "") == 0)))) {
 		return 0;
 	}
 
 	if (farm_is_ingress_mode(f) &&
-		(!f->iethaddr || strcmp(f->iethaddr, "") == 0))
+		(!f->iethaddr || strcmp(f->iethaddr, "") == 0) &&
+		(!f->oethaddr || strcmp(f->oethaddr, "") == 0))
 		return 0;
 
 	return 1;
@@ -193,6 +224,11 @@ static void farm_manage_eventd(void)
 	}
 }
 
+int farm_needs_flowtable(struct farm *f)
+{
+	return f->flow_offload;
+}
+
 static int farm_set_netinfo(struct farm *f)
 {
 	syslog(LOG_DEBUG, "%s():%d: farm %s", __FUNCTION__, __LINE__, f->name);
@@ -212,6 +248,11 @@ static int farm_set_netinfo(struct farm *f)
 	if (farm_needs_policies(f))
 		farm_set_ifinfo(f, KEY_IFACE);
 
+	if (farm_needs_flowtable(f)) {
+		farm_set_ifinfo(f, KEY_IFACE);
+		farm_set_ifinfo(f, KEY_OFACE);
+	}
+
 	return 0;
 }
 
@@ -222,7 +263,7 @@ static int farm_set_mark(struct farm *f, int new_value)
 	syslog(LOG_DEBUG, "%s():%d: farm %s old mark %d new mark %d", __FUNCTION__, __LINE__, f->name, old_value, new_value);
 
 	if (f->mode != VALUE_MODE_DNAT && f->mode != VALUE_MODE_SNAT) {
-		syslog(LOG_ERR, "%s():%d: mark for farm %s not available for the current mode %d", __FUNCTION__, __LINE__, f->name, f->mode);
+		syslog(LOG_INFO, "%s():%d: mark for farm %s not available for the current mode %d", __FUNCTION__, __LINE__, f->name, f->mode);
 		return 0;
 	}
 
@@ -308,14 +349,15 @@ static int farm_set_sched(struct farm *f, int new_value)
 	return 0;
 }
 
-static int farm_strim_netface(char *name)
+static int farm_set_persistence(struct farm *f, int new_value)
 {
-	char *ptr;
+	int old_value = f->persistence;
 
-	if ((ptr = strstr(name, ":")) != NULL) {
-		*ptr = '\0';
-		return 1;
-	}
+	syslog(LOG_DEBUG, "%s():%d: farm %s old persistence %d new persistence %d", __FUNCTION__, __LINE__, f->name, old_value, new_value);
+
+	session_s_delete(f, SESSION_TYPE_STATIC);
+
+	f->persistence = new_value;
 
 	return 0;
 }
@@ -357,6 +399,10 @@ static void farm_print(struct farm *f)
 
 	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_FAMILY, obj_print_family(f->family));
 	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_MODE, obj_print_mode(f->mode));
+
+	if (f->mode == VALUE_MODE_STLSDNAT)
+		syslog(LOG_DEBUG,"    [%s] %d", CONFIG_KEY_RESPONSETTL, f->responsettl);
+
 	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_PROTO, obj_print_proto(f->protocol));
 	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_SCHED, obj_print_sched(f->scheduler));
 
@@ -373,6 +419,8 @@ static void farm_print(struct farm *f)
 
 	obj_print_log(f->log, (char *)buf);
 	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_LOG, buf);
+	if (f->logprefix)
+		syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_LOGPREFIX, f->logprefix);
 
 	syslog(LOG_DEBUG,"    [%s] 0x%x", CONFIG_KEY_MARK, f->mark);
 	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_STATE, obj_print_state(f->state));
@@ -380,32 +428,52 @@ static void farm_print(struct farm *f)
 
 	syslog(LOG_DEBUG,"    [%s] %d", CONFIG_KEY_NEWRTLIMIT, f->newrtlimit);
 	syslog(LOG_DEBUG,"    [%s] %d", CONFIG_KEY_NEWRTLIMITBURST, f->newrtlimitbst);
+	if (f->newrtlimit_logprefix)
+		syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_NEWRTLIMIT_LOGPREFIX, f->newrtlimit_logprefix);
+
 	syslog(LOG_DEBUG,"    [%s] %d", CONFIG_KEY_RSTRTLIMIT, f->rstrtlimit);
 	syslog(LOG_DEBUG,"    [%s] %d", CONFIG_KEY_RSTRTLIMITBURST, f->rstrtlimitbst);
+	if (f->rstrtlimit_logprefix)
+		syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_RSTRTLIMIT_LOGPREFIX, f->rstrtlimit_logprefix);
+
 	syslog(LOG_DEBUG,"    [%s] %d", CONFIG_KEY_ESTCONNLIMIT, f->estconnlimit);
+	if (f->estconnlimit_logprefix)
+		syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_ESTCONNLIMIT_LOGPREFIX, f->estconnlimit_logprefix);
+
 	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_TCPSTRICT, obj_print_switch(f->tcpstrict));
+	if (f->tcpstrict_logprefix)
+		syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_TCPSTRICT_LOGPREFIX, f->tcpstrict_logprefix);
+
 	syslog(LOG_DEBUG,"    [%s] %d", CONFIG_KEY_QUEUE, f->queue);
+	syslog(LOG_DEBUG,"    [%s] %s", CONFIG_KEY_FLOWOFFLOAD, obj_print_switch(f->flow_offload));
 
 	syslog(LOG_DEBUG,"    *[total_weight] %d", f->total_weight);
 	syslog(LOG_DEBUG,"    *[total_bcks] %d", f->total_bcks);
 	syslog(LOG_DEBUG,"    *[bcks_available] %d", f->bcks_available);
 	syslog(LOG_DEBUG,"    *[bcks_are_marked] %d", f->bcks_are_marked);
 	syslog(LOG_DEBUG,"    *[bcks_have_port] %d", f->bcks_have_port);
+	syslog(LOG_DEBUG,"    *[bcks_have_srcaddr] %d", f->bcks_have_srcaddr);
 	syslog(LOG_DEBUG,"    *[policies_action] %d", f->policies_action);
 	syslog(LOG_DEBUG,"    *[policies_used] %d", f->policies_used);
+	syslog(LOG_DEBUG,"    *[total_static_sessions] %d", f->total_static_sessions);
+	syslog(LOG_DEBUG,"    *[total_timed_sessions] %d", f->total_timed_sessions);
 	syslog(LOG_DEBUG,"    *[%s] %d", CONFIG_KEY_ACTION, f->action);
 	syslog(LOG_DEBUG,"    *[reload_action] %x", f->reload_action);
+	syslog(LOG_DEBUG,"    *[nft_chains] %x", f->nft_chains);
 
 	if (f->total_bcks != 0)
 		backend_s_print(f);
 
+	if (f->total_static_sessions || f->total_timed_sessions)
+		session_s_print(f);
+
 	farmpolicy_s_print(f);
 }
 
 static int farm_set_newrtlimit(struct farm *f, int new_value)
 {
 	if (f->newrtlimit == new_value)
-		return PARSER_IDEM_VALUE;
+		return PARSER_OK;
 
 	if (new_value == 0)
 		f->reload_action |= VALUE_RLD_NEWRTLIMIT_STOP;
@@ -419,7 +487,7 @@ static int farm_set_newrtlimit(struct farm *f, int new_value)
 static int farm_set_rstrtlimit(struct farm *f, int new_value)
 {
 	if (f->rstrtlimit == new_value)
-		return PARSER_IDEM_VALUE;
+		return PARSER_OK;
 
 	if (new_value == 0)
 		f->reload_action |= VALUE_RLD_RSTRTLIMIT_STOP;
@@ -433,7 +501,7 @@ static int farm_set_rstrtlimit(struct farm *f, int new_value)
 static int farm_set_estconnlimit(struct farm *f, int new_value)
 {
 	if (f->estconnlimit == new_value)
-		return PARSER_IDEM_VALUE;
+		return PARSER_OK;
 
 	if (new_value == 0)
 		f->reload_action |= VALUE_RLD_ESTCONNLIMIT_STOP;
@@ -444,6 +512,165 @@ static int farm_set_estconnlimit(struct farm *f, int new_value)
 	return PARSER_OK;
 }
 
+static int farm_set_tcpstrict(struct farm *f, int new_value)
+{
+	if (new_value == VALUE_SWITCH_OFF)
+		f->reload_action |= VALUE_RLD_TCPSTRICT_STOP;
+	else
+		f->reload_action |= VALUE_RLD_TCPSTRICT_START;
+
+	f->tcpstrict = new_value;
+	return PARSER_OK;
+}
+
+static int farm_set_queue(struct farm *f, int new_value)
+{
+	if (new_value == -1)
+		f->reload_action |= VALUE_RLD_QUEUE_STOP;
+	else
+		f->reload_action |= VALUE_RLD_QUEUE_START;
+
+	f->queue = new_value;
+	return PARSER_OK;
+}
+
+int farm_changed(struct config_pair *c)
+{
+	struct farm *f = obj_get_current_farm();
+
+	if (!f)
+		return -1;
+
+	syslog(LOG_DEBUG, "%s():%d: farm %s with param %d", __FUNCTION__, __LINE__, f->name, c->key);
+
+	switch (c->key) {
+	case KEY_NAME:
+		return 1;
+		break;
+	case KEY_NEWNAME:
+		return !obj_equ_attribute_string(f->name, c->str_value);
+		break;
+	case KEY_FQDN:
+		return !obj_equ_attribute_string(f->fqdn, c->str_value);
+		break;
+	case KEY_IFACE:
+		return !obj_equ_attribute_string(f->iface, c->str_value);
+		break;
+	case KEY_OFACE:
+		return !obj_equ_attribute_string(f->oface, c->str_value);
+		break;
+	case KEY_ETHADDR:
+		return !obj_equ_attribute_string(f->iethaddr, c->str_value);
+		break;
+	case KEY_FAMILY:
+		return !obj_equ_attribute_int(f->family, c->int_value);
+		break;
+	case KEY_VIRTADDR:
+		return !obj_equ_attribute_string(f->virtaddr, c->str_value);
+		break;
+	case KEY_VIRTPORTS:
+		return !obj_equ_attribute_string(f->virtports, c->str_value);
+		break;
+	case KEY_SRCADDR:
+		return !obj_equ_attribute_string(f->srcaddr, c->str_value);
+		break;
+	case KEY_MODE:
+		return !obj_equ_attribute_int(f->mode, c->int_value);
+		break;
+	case KEY_RESPONSETTL:
+		return !obj_equ_attribute_int(f->responsettl, c->int_value);
+		break;
+	case KEY_PROTO:
+		return !obj_equ_attribute_int(f->protocol, c->int_value);
+		break;
+	case KEY_SCHED:
+		return !obj_equ_attribute_int(f->scheduler, c->int_value);
+		break;
+	case KEY_SCHEDPARAM:
+		return !obj_equ_attribute_int(f->schedparam, c->int_value);
+		break;
+	case KEY_PERSISTENCE:
+		return !obj_equ_attribute_int(f->persistence, c->int_value);
+		break;
+	case KEY_PERSISTTM:
+		return !obj_equ_attribute_int(f->persistttl, c->int_value);
+		break;
+	case KEY_HELPER:
+		return !obj_equ_attribute_int(f->helper, c->int_value);
+		break;
+	case KEY_LOG:
+		return !obj_equ_attribute_int(f->log, c->int_value);
+		break;
+	case KEY_LOGPREFIX:
+		return !obj_equ_attribute_string(f->logprefix, c->str_value);
+		break;
+	case KEY_MARK:
+		return !obj_equ_attribute_int(f->mark, c->int_value);
+		break;
+	case KEY_STATE:
+		return !obj_equ_attribute_int(f->state, c->int_value);
+		break;
+	case KEY_ACTION:
+		return !obj_equ_attribute_int(f->action, c->int_value);
+		break;
+	case KEY_NEWRTLIMIT:
+		return !obj_equ_attribute_int(f->newrtlimit, c->int_value);
+		break;
+	case KEY_NEWRTLIMITBURST:
+		return !obj_equ_attribute_int(f->newrtlimitbst, c->int_value);
+		break;
+	case KEY_RSTRTLIMIT:
+		return !obj_equ_attribute_int(f->rstrtlimit, c->int_value);
+		break;
+	case KEY_RSTRTLIMITBURST:
+		return !obj_equ_attribute_int(f->rstrtlimitbst, c->int_value);
+		break;
+	case KEY_ESTCONNLIMIT:
+		return !obj_equ_attribute_int(f->estconnlimit, c->int_value);
+		break;
+	case KEY_TCPSTRICT:
+		return !obj_equ_attribute_int(f->tcpstrict, c->int_value);
+		break;
+	case KEY_QUEUE:
+		return !obj_equ_attribute_int(f->queue, c->int_value);
+		break;
+	case KEY_FLOWOFFLOAD:
+		return !obj_equ_attribute_int(f->flow_offload, c->int_value);
+		break;
+	case KEY_NEWRTLIMIT_LOGPREFIX:
+		return !obj_equ_attribute_string(f->newrtlimit_logprefix, c->str_value);
+		break;
+	case KEY_RSTRTLIMIT_LOGPREFIX:
+		return !obj_equ_attribute_string(f->rstrtlimit_logprefix, c->str_value);
+		break;
+	case KEY_ESTCONNLIMIT_LOGPREFIX:
+		return !obj_equ_attribute_string(f->estconnlimit_logprefix, c->str_value);
+		break;
+	case KEY_TCPSTRICT_LOGPREFIX:
+		return !obj_equ_attribute_string(f->tcpstrict_logprefix, c->str_value);
+		break;
+	default:
+		break;
+	}
+
+	return 0;
+}
+
+int farm_set_priority(struct farm *f, int new_value)
+{
+	int old_value = f->priority;
+
+	syslog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
+	       __FUNCTION__, __LINE__, old_value, new_value);
+
+	if (new_value <= 0)
+		return -1;
+
+	f->priority = new_value;
+
+	return 0;
+}
+
 void farm_s_print(void)
 {
 	struct list_head *farms = obj_get_farms();
@@ -489,14 +716,15 @@ int farm_set_ifinfo(struct farm *f, int key)
 
 	syslog(LOG_DEBUG, "%s():%d: farm %s set interface info for interface key %d", __FUNCTION__, __LINE__, f->name, key);
 
-	if (!(farm_is_ingress_mode(f) || (farm_needs_policies(f) && key == KEY_IFACE))) {
-		syslog(LOG_DEBUG, "%s():%d: farm %s is not in ingress mode", __FUNCTION__, __LINE__, f->name);
+	if (!(farm_is_ingress_mode(f) ||
+		(farm_needs_policies(f) && key == KEY_IFACE) ||
+		farm_needs_flowtable(f))) {
+		syslog(LOG_DEBUG, "%s():%d: farm %s doesn't require netinfo", __FUNCTION__, __LINE__, f->name);
 		return 0;
 	}
 
 	switch (key) {
 	case KEY_IFACE:
-
 		ret = net_get_local_ifname_per_vip(f->virtaddr, if_str);
 
 		if (ret != 0) {
@@ -505,7 +733,7 @@ int farm_set_ifinfo(struct farm *f, int key)
 		}
 
 		obj_set_attribute_string(if_str, &f->iface);
-		farm_strim_netface(f->iface);
+		net_strim_netface(f->iface);
 
 		if_index = if_nametoindex(f->iface);
 
@@ -515,23 +743,36 @@ int farm_set_ifinfo(struct farm *f, int key)
 		}
 
 		f->ifidx = if_index;
-
 		ether_addr = &f->iethaddr;
 
 		net_get_local_ifinfo((unsigned char **)&ether, f->iface);
-		farm_strim_netface(f->iface);
+		net_strim_netface(f->iface);
+
+		/* By default, use the same inbound and outbound interface until
+		 * the backends network configuration says a different thing */
+		if (f->ofidx == DEFAULT_IFIDX) {
+			f->ofidx = f->ifidx;
+			obj_set_attribute_string(f->iface, &f->oface);
+		}
 
 		sprintf(streth, "%02x:%02x:%02x:%02x:%02x:%02x", ether[0],
 			ether[1], ether[2], ether[3], ether[4], ether[5]);
 
 		obj_set_attribute_string(streth, ether_addr);
 		break;
+
 	case KEY_OFACE:
+		if (f->oface && strcmp(f->oface, IFACE_LOOPBACK) == 0) {
+			syslog(LOG_DEBUG, "%s():%d: farm %s doesn't require output netinfo, loopback interface", __FUNCTION__, __LINE__, f->name);
+			f->ofidx = 0;
+			return 0;
+		}
+
 		ether_addr = &f->oethaddr;
 
 		b = backend_get_first(f);
 		if (!b || b->ipaddr == DEFAULT_IPADDR) {
-			syslog(LOG_ERR, "%s():%d: there is no backend yet in the farm %s", __FUNCTION__, __LINE__, f->name);
+			syslog(LOG_DEBUG, "%s():%d: there is no backend yet in the farm %s", __FUNCTION__, __LINE__, f->name);
 			return 0;
 		}
 
@@ -549,16 +790,14 @@ int farm_set_ifinfo(struct farm *f, int key)
 		}
 
 		obj_set_attribute_string(if_str, &f->oface);
-		farm_strim_netface(f->oface);
+		net_strim_netface(f->oface);
 
-		if_index = if_nametoindex(f->oface);
+		net_get_local_ifinfo((unsigned char **)&ether, f->oface);
+		sprintf(streth, "%02x:%02x:%02x:%02x:%02x:%02x", ether[0],
+			ether[1], ether[2], ether[3], ether[4], ether[5]);
 
-		if (if_index == 0) {
-			syslog(LOG_ERR, "%s():%d: index of outbound interface %s in farm %s is not found", __FUNCTION__, __LINE__, f->oface, f->name);
-			return -1;
-		}
+		obj_set_attribute_string(streth, ether_addr);
 
-		f->ofidx = if_index;
 		break;
 	}
 
@@ -567,14 +806,11 @@ int farm_set_ifinfo(struct farm *f, int key)
 
 int farm_pre_actionable(struct config_pair *c)
 {
-	struct obj_config *cur = obj_get_current_object();
-	struct farm *f;
+	struct farm *f = obj_get_current_farm();
 
-	if (!cur->fptr)
+	if (!f)
 		return -1;
 
-	f = cur->fptr;
-
 	syslog(LOG_DEBUG, "%s():%d: pre actionable farm %s with param %d", __FUNCTION__, __LINE__, f->name, c->key);
 
 	switch (c->key) {
@@ -586,29 +822,30 @@ int farm_pre_actionable(struct config_pair *c)
 	case KEY_VIRTPORTS:
 	case KEY_SRCADDR:
 	case KEY_MODE:
+	case KEY_RESPONSETTL:
 	case KEY_PROTO:
 	case KEY_PERSISTENCE:
 	case KEY_PERSISTTM:
+	case KEY_FLOWOFFLOAD:
+	case KEY_LOG:
+	case KEY_HELPER:
 		if (farm_set_action(f, ACTION_STOP))
 			farm_rulerize(f);
 		break;
 	default:
-		return 0;
+		break;
 	}
 
-	return 0;
+	return ACTION_START;
 }
 
 int farm_pos_actionable(struct config_pair *c)
 {
-	struct obj_config *cur = obj_get_current_object();
-	struct farm *f;
+	struct farm *f = obj_get_current_farm();
 
-	if (!cur->fptr)
+	if (!f)
 		return -1;
 
-	f = cur->fptr;
-
 	syslog(LOG_DEBUG, "%s():%d: pos actionable farm %s with param %d", __FUNCTION__, __LINE__, f->name, c->key);
 
 	switch (c->key) {
@@ -620,9 +857,13 @@ int farm_pos_actionable(struct config_pair *c)
 	case KEY_VIRTPORTS:
 	case KEY_SRCADDR:
 	case KEY_MODE:
+	case KEY_RESPONSETTL:
 	case KEY_PROTO:
 	case KEY_PERSISTENCE:
 	case KEY_PERSISTTM:
+	case KEY_FLOWOFFLOAD:
+	case KEY_LOG:
+	case KEY_HELPER:
 		farm_set_action(f, ACTION_START);
 		break;
 	case KEY_STATE:
@@ -637,16 +878,13 @@ int farm_pos_actionable(struct config_pair *c)
 
 int farm_set_attribute(struct config_pair *c)
 {
-	struct obj_config *cur = obj_get_current_object();
-	struct farm *f;
+	struct farm *f = obj_get_current_farm();
 	struct farm *nf;
 	int ret = PARSER_FAILED;
 
-	if (c->key != KEY_NAME && !cur->fptr)
+	if (c->key != KEY_NAME && !f)
 		return PARSER_OBJ_UNKNOWN;
 
-	f = cur->fptr;
-
 	switch (c->key) {
 	case KEY_NAME:
 		f = farm_lookup_by_name(c->str_value);
@@ -655,7 +893,7 @@ int farm_set_attribute(struct config_pair *c)
 			if (!f)
 				return -1;
 		}
-		cur->fptr = f;
+		obj_set_current_farm(f);
 		ret = PARSER_OK;
 		break;
 	case KEY_NEWNAME:
@@ -671,18 +909,19 @@ int farm_set_attribute(struct config_pair *c)
 		break;
 	case KEY_IFACE:
 		ret = obj_set_attribute_string(c->str_value, &f->iface);
-		farm_strim_netface(f->iface);
+		net_strim_netface(f->iface);
 		break;
 	case KEY_OFACE:
 		ret = obj_set_attribute_string(c->str_value, &f->oface);
-		farm_strim_netface(f->oface);
+		net_strim_netface(f->oface);
 		break;
 	case KEY_FAMILY:
 		f->family = c->int_value;
 		ret = PARSER_OK;
 		break;
 	case KEY_ETHADDR:
-		ret = obj_set_attribute_string(c->str_value, &f->iethaddr);
+		ret = obj_set_attribute_string(c->str_value, &f->iethaddr) ||
+			obj_set_attribute_string(c->str_value, &f->oethaddr);
 		break;
 	case KEY_VIRTADDR:
 		ret = obj_set_attribute_string(c->str_value, &f->virtaddr);
@@ -697,6 +936,10 @@ int farm_set_attribute(struct config_pair *c)
 	case KEY_MODE:
 		ret = farm_set_mode(f, c->int_value);
 		break;
+	case KEY_RESPONSETTL:
+		f->responsettl = c->int_value;
+		ret = PARSER_OK;
+		break;
 	case KEY_PROTO:
 		f->protocol = c->int_value;
 		ret = PARSER_OK;
@@ -709,15 +952,14 @@ int farm_set_attribute(struct config_pair *c)
 		ret = PARSER_OK;
 		break;
 	case KEY_PERSISTENCE:
-		f->persistence = c->int_value;
-		ret = PARSER_OK;
+		ret = farm_set_persistence(f, c->int_value);
 		break;
 	case KEY_PERSISTTM:
 		f->persistttl = c->int_value;
 		ret = PARSER_OK;
 		break;
 	case KEY_PRIORITY:
-		f->priority = c->int_value;
+		farm_set_priority(f, c->int_value);
 		ret = PARSER_OK;
 		break;
 	case KEY_HELPER:
@@ -732,7 +974,10 @@ int farm_set_attribute(struct config_pair *c)
 		ret = farm_set_mark(f, c->int_value);
 		break;
 	case KEY_STATE:
-		ret = farm_set_state(f, c->int_value);
+		if (c->int_value != VALUE_STATE_CONFERR)
+			ret = farm_set_state(f, c->int_value);
+		else
+			ret = PARSER_OK;
 		break;
 	case KEY_ACTION:
 		ret = farm_set_action(f, c->int_value);
@@ -755,13 +1000,33 @@ int farm_set_attribute(struct config_pair *c)
 		ret = farm_set_estconnlimit(f, c->int_value);
 		break;
 	case KEY_TCPSTRICT:
-		f->tcpstrict = c->int_value;
+		ret = farm_set_tcpstrict(f, c->int_value);
 		ret = PARSER_OK;
 		break;
 	case KEY_QUEUE:
-		f->queue = c->int_value;
+		ret = farm_set_queue(f, c->int_value);
 		ret = PARSER_OK;
 		break;
+	case KEY_FLOWOFFLOAD:
+		f->flow_offload = c->int_value;
+		farm_set_netinfo(f);
+		ret = PARSER_OK;
+		break;
+	case KEY_LOGPREFIX:
+		ret = obj_set_attribute_string(c->str_value, &f->logprefix);
+		break;
+	case KEY_NEWRTLIMIT_LOGPREFIX:
+		ret = obj_set_attribute_string(c->str_value, &f->newrtlimit_logprefix);
+		break;
+	case KEY_RSTRTLIMIT_LOGPREFIX:
+		ret = obj_set_attribute_string(c->str_value, &f->rstrtlimit_logprefix);
+		break;
+	case KEY_ESTCONNLIMIT_LOGPREFIX:
+		ret = obj_set_attribute_string(c->str_value, &f->estconnlimit_logprefix);
+		break;
+	case KEY_TCPSTRICT_LOGPREFIX:
+		ret = obj_set_attribute_string(c->str_value, &f->tcpstrict_logprefix);
+		break;
 	default:
 		return PARSER_STRUCT_FAILED;
 	}
@@ -788,6 +1053,7 @@ int farm_set_action(struct farm *f, int action)
 	}
 
 	if (f->action > action) {
+		backend_s_gen_priority(f);
 		farm_manage_eventd();
 		f->action = action;
 		farm_set_netinfo(f);
@@ -869,7 +1135,7 @@ int farm_rulerize(struct farm *f)
 		syslog(LOG_INFO, "%s():%d: farm %s won't be rulerized", __FUNCTION__, __LINE__, f->name);
 		if (f->state == VALUE_STATE_UP)
 			farm_set_state(f, VALUE_STATE_CONFERR);
-		return -1;
+		return 0;
 	}
 
 	return nft_rulerize(f);
@@ -877,16 +1143,17 @@ int farm_rulerize(struct farm *f)
 
 int farm_s_rulerize(void)
 {
-	struct farm *f;
+	struct list_head *farms = obj_get_farms();
+	struct farm *f, *next;
 	int ret = 0;
+	int output = 0;
 
 	syslog(LOG_DEBUG, "%s():%d: rulerize everything", __FUNCTION__, __LINE__);
 
-	struct list_head *farms = obj_get_farms();
-
-	list_for_each_entry(f, farms, list) {
-			ret = ret || farm_rulerize(f);
+	list_for_each_entry_safe(f, next, farms, list) {
+		ret = farm_rulerize(f);
+		output = output || ret;
 	}
 
-	return ret;
+	return output;
 }
