@@ -27,6 +27,7 @@
 #include "backends.h"
 #include "policies.h"
 #include "elements.h"
+#include "sessions.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -108,6 +109,174 @@ struct obj_config * obj_get_current_object(void)
 	return &current_obj;
 }
 
+struct farm * obj_get_current_farm(void)
+{
+	return current_obj.fptr;
+}
+
+struct backend * obj_get_current_backend(void)
+{
+	return current_obj.bptr;
+}
+
+struct policy * obj_get_current_policy(void)
+{
+	return current_obj.pptr;
+}
+
+struct farmpolicy * obj_get_current_farmpolicy(void)
+{
+	return current_obj.fpptr;
+}
+
+struct element * obj_get_current_element(void)
+{
+	return current_obj.eptr;
+}
+
+struct session * obj_get_current_session(void)
+{
+	return current_obj.sptr;
+}
+
+void obj_set_current_farm(struct farm *f)
+{
+	current_obj.fptr = f;
+}
+
+void obj_set_current_backend(struct backend *b)
+{
+	current_obj.bptr = b;
+}
+
+void obj_set_current_policy(struct policy *p)
+{
+	current_obj.pptr = p;
+}
+
+void obj_set_current_farmpolicy(struct farmpolicy *fp)
+{
+	current_obj.fpptr = fp;
+}
+
+void obj_set_current_element(struct element *e)
+{
+	current_obj.eptr = e;
+}
+
+void obj_set_current_session(struct session *s)
+{
+	current_obj.sptr = s;
+}
+
+char * obj_print_key(int key)
+{
+	switch (key) {
+	case KEY_FARMS:
+		return CONFIG_KEY_FARMS;
+	case KEY_NAME:
+		return CONFIG_KEY_NAME;
+	case KEY_NEWNAME:
+		return CONFIG_KEY_NEWNAME;
+	case KEY_FQDN:
+		return CONFIG_KEY_FQDN;
+	case KEY_IFACE:
+		return CONFIG_KEY_IFACE;
+	case KEY_OFACE:
+		return CONFIG_KEY_OFACE;
+	case KEY_FAMILY:
+		return CONFIG_KEY_FAMILY;
+	case KEY_ETHADDR:
+		return CONFIG_KEY_ETHADDR;
+	case KEY_VIRTADDR:
+		return CONFIG_KEY_VIRTADDR;
+	case KEY_VIRTPORTS:
+		return CONFIG_KEY_VIRTPORTS;
+	case KEY_IPADDR:
+		return CONFIG_KEY_IPADDR;
+	case KEY_SRCADDR:
+		return CONFIG_KEY_SRCADDR;
+	case KEY_PORT:
+		return CONFIG_KEY_PORT;
+	case KEY_MODE:
+		return CONFIG_KEY_MODE;
+	case KEY_RESPONSETTL:
+		return CONFIG_KEY_RESPONSETTL;
+	case KEY_PROTO:
+		return CONFIG_KEY_PROTO;
+	case KEY_SCHED:
+		return CONFIG_KEY_SCHED;
+	case KEY_SCHEDPARAM:
+		return CONFIG_KEY_SCHEDPARAM;
+	case KEY_PERSISTENCE:
+		return CONFIG_KEY_PERSIST;
+	case KEY_PERSISTTM:
+		return CONFIG_KEY_PERSISTTM;
+	case KEY_HELPER:
+		return CONFIG_KEY_HELPER;
+	case KEY_LOG:
+		return CONFIG_KEY_LOG;
+	case KEY_LOGPREFIX:
+		return CONFIG_KEY_LOGPREFIX;
+	case KEY_MARK:
+		return CONFIG_KEY_MARK;
+	case KEY_STATE:
+		return CONFIG_KEY_STATE;
+	case KEY_BCKS:
+		return CONFIG_KEY_BCKS;
+	case KEY_WEIGHT:
+		return CONFIG_KEY_WEIGHT;
+	case KEY_PRIORITY:
+		return CONFIG_KEY_PRIORITY;
+	case KEY_ACTION:
+		return CONFIG_KEY_ACTION;
+	case KEY_NEWRTLIMIT:
+		return CONFIG_KEY_NEWRTLIMIT;
+	case KEY_NEWRTLIMITBURST:
+		return CONFIG_KEY_NEWRTLIMITBURST;
+	case KEY_NEWRTLIMIT_LOGPREFIX:
+		return CONFIG_KEY_NEWRTLIMIT_LOGPREFIX;
+	case KEY_RSTRTLIMIT:
+		return CONFIG_KEY_RSTRTLIMIT;
+	case KEY_RSTRTLIMITBURST:
+		return CONFIG_KEY_RSTRTLIMITBURST;
+	case KEY_RSTRTLIMIT_LOGPREFIX:
+		return CONFIG_KEY_RSTRTLIMIT_LOGPREFIX;
+	case KEY_ESTCONNLIMIT:
+		return CONFIG_KEY_ESTCONNLIMIT;
+	case KEY_ESTCONNLIMIT_LOGPREFIX:
+		return CONFIG_KEY_ESTCONNLIMIT_LOGPREFIX;
+	case KEY_TCPSTRICT:
+		return CONFIG_KEY_TCPSTRICT;
+	case KEY_TCPSTRICT_LOGPREFIX:
+		return CONFIG_KEY_TCPSTRICT_LOGPREFIX;
+	case KEY_QUEUE:
+		return CONFIG_KEY_QUEUE;
+	case KEY_FLOWOFFLOAD:
+		return CONFIG_KEY_FLOWOFFLOAD;
+	case KEY_POLICIES:
+		return CONFIG_KEY_POLICIES;
+	case KEY_TYPE:
+		return CONFIG_KEY_TYPE;
+	case KEY_TIMEOUT:
+		return CONFIG_KEY_TIMEOUT;
+	case KEY_ELEMENTS:
+		return CONFIG_KEY_ELEMENTS;
+	case KEY_DATA:
+		return CONFIG_KEY_DATA;
+	case KEY_TIME:
+		return CONFIG_KEY_TIME;
+	case KEY_SESSIONS:
+		return CONFIG_KEY_SESSIONS;
+	case KEY_CLIENT:
+		return CONFIG_KEY_CLIENT;
+	case KEY_BACKEND:
+		return CONFIG_KEY_BACKEND;
+	default:
+		return NULL;
+	}
+}
+
 char * obj_print_family(int family)
 {
 	switch (family) {
@@ -133,6 +302,8 @@ char * obj_print_mode(int mode)
 		return CONFIG_VALUE_MODE_DSR;
 	case VALUE_MODE_STLSDNAT:
 		return CONFIG_VALUE_MODE_STLSDNAT;
+	case VALUE_MODE_LOCAL:
+		return CONFIG_VALUE_MODE_LOCAL;
 	default:
 		return NULL;
 	}
@@ -291,28 +462,43 @@ char * obj_print_switch(int state)
 
 int obj_set_attribute(struct config_pair *c, int actionable)
 {
-	int ret;
-	int status;
+	int ret = 0;
+	int action = ACTION_NONE;
 	syslog(LOG_DEBUG, "%s():%d: actionable is %d", __FUNCTION__, __LINE__, actionable);
 
 	switch (c->level) {
 	case LEVEL_FARMS:
+		if (!farm_changed(c))
+			return PARSER_OK;
+
 		if (actionable)
-			farm_pre_actionable(c);
+			action = farm_pre_actionable(c);
 
 		ret = farm_set_attribute(c);
 
-		if (actionable && ret == PARSER_OK)
+		if (actionable && ret == PARSER_OK && action != ACTION_NONE)
 			farm_pos_actionable(c);
 		break;
 	case LEVEL_BCKS:
+		if (!backend_changed(c))
+			return PARSER_OK;
+
 		if (actionable)
-			status = bck_pre_actionable(c);
+			action = bck_pre_actionable(c);
 
 		ret = backend_set_attribute(c);
 
-		if (actionable && status == 0)
-			bck_pos_actionable(c);
+		if (actionable && action != ACTION_NONE)
+			bck_pos_actionable(c, action);
+		break;
+	case LEVEL_SESSIONS:
+		if (actionable)
+			session_pre_actionable(c);
+
+		ret = session_set_attribute(c);
+
+		if (actionable)
+			session_pos_actionable(c);
 		break;
 	case LEVEL_FARMPOLICY:
 		if (actionable)
@@ -360,17 +546,41 @@ int obj_set_attribute_string(char *src, char **dst)
 	return 0;
 }
 
+int obj_equ_attribute_string(char *stra, char *strb)
+{
+	return (stra == strb ||
+			(stra == NULL && strcmp(strb, "") == 0) ||
+			(stra && strb && strcmp(stra, strb) == 0));
+}
+
+void obj_set_attribute_int(int *src, int value)
+{
+	*src = value;
+}
+
+int obj_equ_attribute_int(int valuea, int valueb)
+{
+	return valuea == valueb;
+}
+
 void obj_print(void)
 {
 	farm_s_print();
 	policies_s_print();
 }
 
-int obj_rulerize(void)
+int obj_rulerize(int mode)
 {
+	int out = 0;
 	obj_config_init();
-	policy_s_rulerize();
-	return farm_s_rulerize();
+	if (mode == OBJ_START_INV) {
+		out = farm_s_rulerize();
+		out = out + policy_s_rulerize();
+	} else {
+		out = policy_s_rulerize();
+		out = out + farm_s_rulerize();
+	}
+	return out;
 }
 
 char * obj_print_policy_type(int type)
